#########################################
#          Conilurus Genomics           #
#########################################

# Clear environment and set up working dir and datestamp
rm(list = ls())
setwd("E:/conilurus_genetics")
currentDate <- Sys.Date() # Add current date to environment for output of file names with date
options(scipen = 999) #prevent display in scientific notation





####################################################################################################
#                             Depth checking after angsd run                                       #
####################################################################################################

# Sort out a global depth count file
depth <- read.table("output/coni_070820.depthGlobal")
depth <- as.data.frame(t(depth))
depth[1:10,]
colnames(depth) <- "nosites"
rownames(depth) <- seq(1:nrow(depth))
depth$readspersite <- as.numeric(rownames(depth))-1
depth$noreads <- depth$nosites*depth$readspersite
depth[1:10,1:3]
depth[800:810,1:3]
nrow(depth)
depth[59990:60001,1:3]
#depth <- depth[1:100000,] # Not necessary to remove the final row
hist(depth$noreads,ylim=c(0,2000),xlim=c(0,200000),breaks=50)
meancoverage <- sum(depth$noreads)/sum(depth$nosites)/66 # num samps
meancoverage


# Now use a loop to create a df with a row for every site, with it's number of reads in the adjacent column
bysite <- as.data.frame(rep(NA, sum(depth$nosites)))
colnames(bysite) <- "site"
bysite$site <- seq(1,nrow(bysite),1)
bysite$noreads <- NA
bysite[1:10,]

depth <- depth[depth$nosites>0,]
sitelist <- list()
i <- 1
while(i <= nrow(depth)) {
  sitelist[[i]] <- rep(depth$readspersite[i],depth$nosites[i])
  i <- i + 1
}
sitelist[1:10]

singlevec <- Reduce(c,sitelist)

bysite$noreads <- singlevec
bysite[1:10,]

sum(bysite$noreads) # Total number of reads
mean(bysite$noreads)/66 # This should be mean reads/site/sample
median(bysite$noreads)/66 # This should be median reads/site/sample



# using the per sample depth file
rm(list = ls())
depth <- as.data.frame(read.table("output/coni_290720.depthSample"))
depth <- depth[1:66,1:20000] # This removes a bunch of zeros
depth[1:20,1:20]
v2 <- depth$V2
depth2 <- depth
for(i in 2:ncol(depth2)){
  depth2[,i] <- depth2[,i]*ncol(depth2[,1:(i-1)])
}
depth2$V2 <- v2
depth2[1:20,1:20]
depth2$readspersite <- (rowSums(depth2[,2:ncol(depth2)]))/(rowSums(depth))
hist(depth2$readspersite)
coverage <- mean(depth2$readspersite,na.rm=TRUE)
coverage # should be similar to mean reads/site/sample calculated above




########################################################################

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()




################################
## ALL SAMPLES INITIAL FILTER ##
################################

## Filter to good SNPs and samples dataset

# Read in snp sample matrix from ANGSD
conigens <- read.table(file="output/coni_070820.geno", header=FALSE, row.names = NULL, na.strings = -1) # 66 samples (incl 5 replicates)

# Sort out formatting
conigens$V1 <- paste(conigens$V1,"-",conigens$V2, sep="")
conigens$V2 <- NULL
rownames(conigens) <- conigens$V1
conigens$V1 <- NULL

samps <- read.csv(file="output/samp_info_070820.csv",header=TRUE,row.names=NULL) # 66 samples
colnames(conigens) <- samps$sample
conigens[1:8,1:8]



##############################################################################
### Iteratively remove samples,snps,samples,snps,samples (O'Leary et al. 2018)

## Remove some really crappy samples before filtering loci
conigens <- as.data.frame(t(conigens))
nrow(conigens) # number of samples before filter
hist(rowSums(is.na(conigens)))
conigens <- conigens[rowSums(is.na(conigens)) < 0.98*ncol(conigens), ] # Unavoidable loss of samples due to missingness

## Filter out really crappy SNPs, based on callrate
# Manually calculate callrate
conigens <- as.data.frame(t(conigens))
conigens$missingperloc <- rowSums(is.na(conigens))
conigens$samplestyped <- ncol(conigens[1:(ncol(conigens)-1)])-conigens$missingperloc
conigens$callrate <- conigens$samplestyped/(conigens$samplestyped+conigens$missingperloc)
conigens <- conigens[conigens$callrate > 0.85,]

## Remove more crap samples samples before filtering loci again
conigens <- as.data.frame(t(conigens))
conigens <- conigens[rowSums(is.na(conigens)) < 0.85*ncol(conigens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
conigens <- as.data.frame(t(conigens))
conigens$missingperloc <- rowSums(is.na(conigens))
conigens$samplestyped <- ncol(conigens[1:(ncol(conigens)-1)])-conigens$missingperloc
conigens$callrate <- conigens$samplestyped/(conigens$samplestyped+conigens$missingperloc)
conigens <- conigens[conigens$callrate > 0.88,]

# Remove more samples
conigens <- as.data.frame(t(conigens))
conigens <- conigens[rowSums(is.na(conigens)) < 0.75*ncol(conigens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
conigens <- as.data.frame(t(conigens))
conigens$missingperloc <- rowSums(is.na(conigens))
conigens$samplestyped <- ncol(conigens[1:(ncol(conigens)-1)])-conigens$missingperloc
conigens$callrate <- conigens$samplestyped/(conigens$samplestyped+conigens$missingperloc)
conigens <- conigens[conigens$callrate > 0.9,]

# Remove more samples
conigens <- as.data.frame(t(conigens))
conigens <- conigens[rowSums(is.na(conigens)) < 0.65*ncol(conigens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
conigens <- as.data.frame(t(conigens))
conigens$missingperloc <- rowSums(is.na(conigens))
conigens$samplestyped <- ncol(conigens[1:(ncol(conigens)-1)])-conigens$missingperloc
conigens$callrate <- conigens$samplestyped/(conigens$samplestyped+conigens$missingperloc)
conigens <- conigens[conigens$callrate > 0.92,]

# Remove more samples
conigens <- as.data.frame(t(conigens))
conigens <- conigens[rowSums(is.na(conigens)) < 0.5*ncol(conigens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
conigens <- as.data.frame(t(conigens))
conigens$missingperloc <- rowSums(is.na(conigens))
conigens$samplestyped <- ncol(conigens[1:(ncol(conigens)-1)])-conigens$missingperloc
conigens$callrate <- conigens$samplestyped/(conigens$samplestyped+conigens$missingperloc)
conigens <- conigens[conigens$callrate > 0.935,]

# Remove more samples
conigens <- as.data.frame(t(conigens))
conigens <- conigens[rowSums(is.na(conigens)) < 0.4*ncol(conigens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
conigens <- as.data.frame(t(conigens))
conigens$missingperloc <- rowSums(is.na(conigens))
conigens$samplestyped <- ncol(conigens[1:(ncol(conigens)-1)])-conigens$missingperloc
conigens$callrate <- conigens$samplestyped/(conigens$samplestyped+conigens$missingperloc)
conigens <- conigens[conigens$callrate > 0.94,]

# Remove more samples
conigens <- as.data.frame(t(conigens))
conigens <- conigens[rowSums(is.na(conigens)) < 0.3*ncol(conigens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
conigens <- as.data.frame(t(conigens))
conigens$missingperloc <- rowSums(is.na(conigens))
conigens$samplestyped <- ncol(conigens[1:(ncol(conigens)-1)])-conigens$missingperloc
conigens$callrate <- conigens$samplestyped/(conigens$samplestyped+conigens$missingperloc)
conigens <- conigens[conigens$callrate > 0.95,]

# Remove more samples
conigens <- as.data.frame(t(conigens))
conigens <- conigens[rowSums(is.na(conigens)) < 0.2*ncol(conigens), ] # Unavoidable loss of samples due to missingness




## Filter to good SNPs, based on callrate and MAF
# Manually calculate callrate, missingness, minor allele frequency
conigens <- as.data.frame(t(conigens))
conigens$missingperloc <- rowSums(is.na(conigens))
conigens$samplestyped <- ncol(conigens[1:(ncol(conigens)-1)])-conigens$missingperloc
conigens$allelecount <- rowSums(conigens[1:(ncol(conigens)-2)],na.rm=TRUE)
conigens$alf <- conigens$allelecount/(2*conigens$samplestyped)
conigens$alleleA <- 1-conigens$alf
conigens$alf <- ifelse(conigens$alf>0.5, conigens$alleleA, conigens$alf)
conigens$alleleA <- 1-conigens$alf

# Just changing name to avoid reading data in again and again and again and again
congen <- conigens

# Filter on Minor Allele Frequency
# Can filter on the number of times a minor allele is present rather than the MAF (Roesti et al. 2012)
congen <- congen[congen$allelecount>=3,]
# congen <- congen[congen$alf>=0.05,]

# Calculate Hexp and Hobs and only keep all loci with Ho <=0.x
# Hexp = 2pq = 1 - (p^2 + q^2)
congen$Hs <- 2*congen$alf*congen$alleleA
congen$Ho <- as.numeric(rowSums(congen[1:(ncol(conigens)-7)]==1,na.rm=TRUE))
congen$Ho <- congen$Ho/congen$samplestyped



# Can check the influence of FIS filter on Hobs and MAF patterns
# FIS = 1 - (HO/HS)
# congen$fis <- 1-(congen$Ho/congen$Hs)
# congen <- congen[congen$fis<0.5,]


# Drop if Hobs is > 0.x
congen <- congen[congen$Ho<0.5,]


# Check patterns in SNPs
plot(congen$alf)
hist(congen$alf)
plot(congen$Hs,congen$Ho)
hist(congen$Ho)


# Remove extraneous columns
head(congen[,(ncol(congen)-9):(ncol(congen))]) # shows head of last x columns
congen <- congen[,1:(ncol(congen)-8)] # Removes last x columns
head(congen[,(ncol(congen)-9):(ncol(congen))]) # All good





#### Filter on linkage

## Create new df for filtering
congen.linkage <- congen
congen.linkage[1:6,1:6]
congen.linkage$locus <- rownames(congen.linkage)

allloci <- read.csv("output/all_loci_coni_290720.csv", header=TRUE)
congen.linkage <- merge(congen.linkage, allloci, all.x=TRUE, by="locus")
head(congen.linkage[ , (ncol(congen.linkage)-4) : (ncol(congen.linkage))]) # Check last columns

# source("https://bioconductor.org/biocLite.R")
# biocLite("SNPRelate")
library("gdsfmt")
library(SNPRelate)
library("ramify")

## Sort out a gds file for SNPrelate
genmat <- congen.linkage[, 2:(ncol(congen.linkage)-2)] # create SNP only matrix (i.e. remove chrom columns)
genmat[] <- lapply(genmat, function(x) {
  if(is.factor(x)) as.numeric(as.character(x)) else x
})
sapply(genmat, class)
genmat <- as.matrix(genmat)
SampID <- colnames(congen.linkage)[2:(ncol(congen.linkage)-2)] # Get a list of samples
SNP_ID <- congen.linkage[, 1] # Get a list of SNPs
Chrom <- as.integer(as.factor(congen.linkage[, ncol(congen.linkage)-1])) # Get a list of chromosomes corresponding to SNPs
ChromPos <- congen.linkage[, ncol(congen.linkage)] # Get a list of chromosome positions corresponding to SNPs

# Make a GDS dataset from the SNP x Sample matrix
# gds <- snpgdsCreateGeno("output/LinkFilt300720.gds", genmat = genmat, sample.id = SampID, snp.id = SNP_ID, snp.chromosome = Chrom, snp.position = ChromPos, snpfirstdim = TRUE)
gds <- snpgdsCreateGeno("output/LinkFilt140920.gds", genmat = genmat, sample.id = SampID, snp.id = SNP_ID, snp.chromosome = Chrom, snp.position = ChromPos, snpfirstdim = TRUE)
# gds <- snpgdsOpen("output/LinkFilt300720.gds")
gds <- snpgdsOpen("output/LinkFilt140920.gds")

# LD pruning - Try different LD thresholds for sensitivity analysis
set.seed(32) # Hopefully this ensure the filtering produces the exact same SNPs each time it's redone
gds_LD <- snpgdsLDpruning(gds, method="corr", ld.threshold = 0.5, slide.max.bp = 100000, autosome.only = FALSE)
# (https://journals.plos.org/plosgenetics/article?id=10.1371/journal.pgen.0030144) suggests strong associations over no more than 100kb

# How many SNPs have we filtered down to?
LDFilt <- unlist(gds_LD)
length(LDFilt)


## Subset main data frame using independent SNPs - better for relatedness and PCA
congen <- subset(congen, rownames(congen) %in% LDFilt)
congen <- as.data.frame(t(congen))

# Check samples
nsamps <- nrow(congen)
nsamps

# Check snps
nsnps <- ncol(congen)
nsnps





## Get population names
samps <- subset(samps,samps$sample %in% rownames(congen))
identical(samps$sample,rownames(congen))
popns <- samps$popn
table(popns)



# Close the stupid gds file
snpgdsClose(gds)
detach("package:SNPRelate", unload=TRUE)



########################################################################################################


# Output SNP Locus names for future reference
write.csv(colnames(congen),paste("output/",length(LDFilt),"_snp_IDs_",currentDate,".csv", sep=""),row.names = FALSE)







####################################################################
# Create hierarchical clustering dendrogram to visualise structure #
####################################################################

# Change name to avoid filtering if you mess up somewhere
geno.types <- congen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# ## If you want to do population level dendrogram - do this
# # Convert to genpop object
# genotypes <- genind2genpop(genotypes, quiet = FALSE, process.other = TRUE,
#                            other.action = mean)
# 
# # Get population genetic distance (1=nei, 4=euclidean)
# genotypes <- dist.genpop(genotypes, method=1)
# hc <- hclust(genotypes)
# plot(as.phylo(hc),cex=1,show.tip.label=TRUE)

## Produce distance matrix and dendrogram
hc <- hclust(diss.dist(genotypes, percent = FALSE, mat = FALSE))
plot(hc, labels=NULL)
# Tech Reps and Biol Reps are paired up very strongly in this plot. So not likely to be much influence
# from different runs/lanes/plates.
plot(hc, labels=genotypes@pop) # Same but with pops as names

# Plot as a phylogeny
library(ape)
plot(as.phylo(hc),cex=1,show.tip.label=TRUE)

# Output for manuscript supplementary material
pdffilename <- paste("output/coni_dendro_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=6, height=12)
plot(as.phylo(hc),cex=1,show.tip.label=TRUE)
dev.off()






## Remove technical replicates
congen$popn <- popns
remove <- c("CPE0006x","CPE0049x","CPE0051x","CPE0011x","CPE0018x")
congen <- congen[!rownames(congen) %in% remove, ]



## Output dataset to use for analyses
popns <- congen$popn
congen$popn <- NULL
write.csv(as.data.frame(t(congen)),"output/congen140920.csv",row.names = TRUE, quote = FALSE)
write.csv(popns, "output/congen_popns_140920.csv", row.names = FALSE)









#########################################
# Check relatedness between individuals #
#########################################

# Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))

### Relatedness using SNPRelate package
## Create new df for filtering
congenrelate <- as.data.frame(t(congen))
congenrelate[1:6,1:6]
congenrelate$locus <- rownames(congenrelate)
allloci <- read.csv("output/all_loci_coni_290720.csv", header=TRUE)
congenrelate <- merge(congenrelate, allloci, all.x=TRUE, by="locus")
head(congenrelate[ , (ncol(congenrelate)-4) : (ncol(congenrelate))]) # Check last columns

# source("https://bioconductor.org/biocLite.R")
# biocLite("SNPRelate")
library("gdsfmt")
library(SNPRelate)
library("ramify")

## Sort out a gds file for SNPrelate
genmat <- congenrelate[, 2:(ncol(congenrelate)-2)] # create SNP only matrix (i.e. remove chrom columns)
genmat[] <- lapply(genmat, function(x) {
  if(is.factor(x)) as.numeric(as.character(x)) else x
})
sapply(genmat, class)
genmat <- as.matrix(genmat)
SampID <- colnames(congenrelate)[2:(ncol(congenrelate)-2)] # Get a list of samples
SNP_ID <- congenrelate[, 1] # Get a list of SNPs
Chrom <- as.integer(as.factor(congenrelate[, ncol(congenrelate)-1])) # Get a list of chromosomes corresponding to SNPs
ChromPos <- congenrelate[, ncol(congenrelate)] # Get a list of chromosome positions corresponding to SNPs

# Make a GDS dataset from the SNP x Sample matrix
gds <- snpgdsCreateGeno("output/Relatedness010221.gds", genmat = genmat, sample.id = SampID, snp.id = SNP_ID, snp.chromosome = Chrom, snp.position = ChromPos, snpfirstdim = TRUE)
gds <- snpgdsOpen("output/Relatedness010221.gds")

# Make population groups
BI <- c("CPE0011","CPE0012","CPE0014","CPE0015","CPE0016","CPE0017")
CE <- c("CPE0001","CPE0002","CPE0003","CPE0004","CPE0006","CPE0009","CPE0030",
        "CPE0031","CPE0032","CPE0033","CPE0034","CPE0035","CPE0036","CPE0037","CPE0038")
CW <- c("CPE0039","CPE0043","CPE0044")
II <- c("CPE0047","CPE0048","CPE0049","CPE0050")
MI <- c("CPE0018","CPE0019","CPE0020","CPE0022","CPE0023","CPE0024","CPE0025","CPE0026","CPE0027","CPE0028","CPE0029")
MP <- c("CPE0051","CPE0052","CPE0054","CPE0055","CPE0056","CPE0057","CPE0058","CPE0059","CPE0060","CPE0061")
PR <- c("CPE0062","CPE0064")
poops <- list(BI,CE,CW,II,MI,MP,PR)

# Calculate IBD coefficients for each locality
library("otuSummary")
kinships <- as.data.frame(matrix(nrow=3,ncol=3))
colnames(kinships) <- c("ind1","ind2","kinship")

for(i in poops){
  relatedness <- snpgdsIBDMLE(gds,autosome.only = FALSE, kinship=TRUE, method="EM", sample.id = i)  #coeff.correct=TRUE
  kins <- as.data.frame(relatedness$kinship)
  rownames(kins) <- i
  colnames(kins) <- i
  kins
  ks <- matrixConvert(kins, colname = c("ind1", "ind2", "kinship"))
  kinships <- rbind(kinships,ks)
}

# Finalise and output df
kinships <- kinships[4:nrow(kinships),]
kinships[kinships$kinship>0.25,]
write.csv(kinships,"output/kinships010221.csv",row.names=FALSE)




### Relatedness using Weir & Goudet 2017 / Goudet 2018 methods
library(hierfstat)

# Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))
congen[1:10,1:5]

# Make population groups
BI <- c("CPE0011","CPE0012","CPE0014","CPE0015","CPE0016","CPE0017")
CE <- c("CPE0001","CPE0002","CPE0003","CPE0004","CPE0006","CPE0009","CPE0030",
        "CPE0031","CPE0032","CPE0033","CPE0034","CPE0035","CPE0036","CPE0037","CPE0038")
CW <- c("CPE0039","CPE0043","CPE0044")
II <- c("CPE0047","CPE0048","CPE0049","CPE0050")
MI <- c("CPE0018","CPE0019","CPE0020","CPE0022","CPE0023","CPE0024","CPE0025","CPE0026","CPE0027","CPE0028","CPE0029")
MP <- c("CPE0051","CPE0052","CPE0054","CPE0055","CPE0056","CPE0057","CPE0058","CPE0059","CPE0060","CPE0061")
PR <- c("CPE0062","CPE0064")
poops <- list(BI,CE,CW,II,MI,MP,PR)

# Run same loop as above but use the Goudet estimator instead - just runs off the subsetted congen df (instead of stupid gds format)
relatedness <- as.data.frame(matrix(nrow=3,ncol=3))
colnames(relatedness) <- c("ind1","ind2","relatedness")

for(i in poops){
  rels <- subset(congen,rownames(congen) %in% i)
  rels <- beta.dosage(rels,inb=FALSE,Mb=FALSE)
  rels <- matrixConvert(rels, colname = c("ind1", "ind2", "relatedness"))
  relatedness <- rbind(relatedness,rels)
}

# Finalise and output df
relatedness <- relatedness[4:nrow(relatedness),]
write.csv(relatedness,"output/relatedness020221.csv",row.names=FALSE)









#####################################
# Remove highly related individuals #
#####################################

# Clear environment and set up working dir and datestamp
rm(list = ls())
currentDate <- Sys.Date() # Add current date to environment for output of file names with date

# Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))

# Closely related individuals selected in outputs from above
relatives <- c("CPE0018","CPE0022","CPE0023","CPE0025","CPE0026","CPE0024","CPE0028","CPE0009","CPE0031","CPE0038","CPE0044","CPE0048","CPE0050")

# Remove rels
congen$popn <- popns
congen <- subset(congen, !(rownames(congen) %in% relatives))
popns <- congen$popn
congen$popn <- NULL

## Output final dataset to use for analyses
write.csv(as.data.frame(t(congen)),"output/congen040221.csv",row.names = TRUE, quote = FALSE)
write.csv(popns, "output/congen_popns_040221.csv", row.names = FALSE)
rm(relatives)






#########################################################
# Make MDS plot of genetic distance between populations #
#########################################################

# Can read in full or reduced dataset here
## Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))
geno.types <- congen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Check missingness
library(poppr)
div <- summary(genotypes) # 5 mins with large datasets
div$NA.perc # 1.9% missing data - good
rowSums(is.na(geno.types))


# Convert to genpop object
congenpop <- genind2genpop(genotypes, quiet = FALSE, process.other = TRUE,
                                      other.action = mean)

# get individual genetic distance
x <- prevosti.dist(genotypes)

# Get population genetic distance (1=nei, 4=euclidean)
PopnGenDist <- dist.genpop(congenpop, method=1, diag=TRUE, upper=FALSE)

## Classical (metric) MDS plot using population distance matrix
PopnGenDist <- as.matrix(PopnGenDist)
# PopnGenDist <- as.matrix(x) # For individual pcoa
fitt <- cmdscale(PopnGenDist, eig=TRUE, k=2, add=TRUE) # classical MDS of distance plot; k=number of dimensions
round(fitt$eig*100/sum(fitt$eig),1) # Percentage of variance explained by each axis

# Use MDS output (fit) to plot principal coordinates using GGPLOT2
fitt <- as.data.frame(fitt$points) # Needs to be a stupid data.frame not a matrix for ggplot

library(ggrepel) # Can use this to plot with pretty labels that don't touch (repel) each other
library(ggplot2)
mdspopgendist <- ggplot(fitt, aes(V1,V2)) +
  geom_vline(xintercept=0, size=0.3, color="darkgrey") +
  geom_hline(yintercept=0, size=0.3, color="darkgrey") +
  geom_point(aes(shape=as.factor(rownames(fitt))), size=5) +
  scale_shape_manual(values=rep(c(15,16,16,18,15,17,17),3), guide=FALSE) +
  # geom_label_repel(aes(label=rownames(fitt)), label.size = NA, label.padding = 0.5, point.padding = 1.5) +
  geom_text_repel(aes(label=rownames(fitt)), force=0.2, max.iter = 5000, point.padding=0.8, segment.color = "transparent") +
  theme_bw() +
  theme(panel.grid.major.y = element_blank()) +  
  theme(panel.grid.minor.y = element_blank()) +
  theme(panel.grid.major.x = element_blank()) +  
  theme(panel.grid.minor.x = element_blank()) +
  xlab("Coordinate 1") + 
  ylab("Coordinate 2") +
  theme(axis.title.y=element_text(size=13, margin=margin(0,16,0,0))) +
  theme(axis.title.x=element_text(size=13, margin=margin(16,0,0,0))) +
  theme(axis.text = element_text(size = 13)) +
  theme(legend.position="none") +
  theme(panel.border = element_blank()) +
  theme(axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5)) +
  scale_y_continuous(expand=c(0,0), limits=c(-0.09,0.07)) +
  scale_x_continuous(expand=c(0,0), limits=c(-0.32,0.22))
mdspopgendist

# Output MDS to PDF
pdffilename <- paste("output/MDS_pop_gen_distance", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=8, height=8)
mdspopgendist
dev.off()


# Individual MDS
mdsindgendist <- ggplot(fitt, aes(V1,V2, col=as.factor(popns))) +
  geom_vline(xintercept=0, size=0.3, color="darkgrey") +
  geom_hline(yintercept=0, size=0.3, color="darkgrey") +
  geom_point(size=3) +
  scale_color_manual(values=c("red","green3","blue","cyan","magenta","yellow","gray")) +
  theme_bw() +
  theme(panel.grid.major.y = element_blank()) +  
  theme(panel.grid.minor.y = element_blank()) +
  theme(panel.grid.major.x = element_blank()) +  
  theme(panel.grid.minor.x = element_blank()) +
  xlab("Coordinate 1") + 
  ylab("Coordinate 2") +
  theme(axis.title.y=element_text(size=13, margin=margin(0,16,0,0))) +
  theme(axis.title.x=element_text(size=13, margin=margin(16,0,0,0))) +
  theme(axis.text = element_text(size = 13)) +
  theme(panel.border = element_blank()) +
  theme(axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5))
mdsindgendist

# Output MDS to PDF
pdffilename <- paste("output/MDS_ind_gen_distance", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=9, height=8)
mdsindgendist
dev.off()








#######################################################
# Get standard genetic parameters for all populations #
#######################################################

# Remove the small (< 6 individuals) populations first
table(popns)
genotypes <- genotypes[genotypes@pop!="Cobourg West",]
genotypes <- genotypes[genotypes@pop!="Prince Regent",]
genotypes <- genotypes[genotypes@pop!="Inglis Island",]

## Gstudio package
# require(devtools)
# install_github("dyerlab/popgraph")
library(popgraph)
# install_github("dyerlab/gstudio")
library(gstudio)

# Convert data to gstudio format
data4gstudio <- genind2df(genotypes, sep="", usepop=TRUE, oneColPerAll=FALSE)
data4gstudio[data4gstudio==22] <- 2
data4gstudio[data4gstudio==12] <- 1
data4gstudio[data4gstudio==11] <- 0
data4gstudio[1:10,1:10]


# Read into gstudio
write.csv(data4gstudio, "output/gstuddat.csv")
gstudred <- read_population("output/gstuddat.csv", type = "snp", locus.columns = 3:(ncol(data4gstudio)+1))
gstudred[1:10,1:6]


# Group by population
clades <- partition(gstudred, stratum = "pop")
names(clades)
lapply(clades, dim) # looking good

# Get genetic stats by locus for each population
gstua <- genetic_diversity(gstudred, stratum="pop", mode = "A", small.N = TRUE) # This will take forever
gstuae <- genetic_diversity(gstudred, stratum="pop", mode = "Ae", small.N = TRUE) # This will take forever
gstuhe <- genetic_diversity(gstudred, stratum="pop", mode = "He", small.N = TRUE) # This will take forever
gstuho <- genetic_diversity(gstudred, stratum="pop", mode = "Ho", small.N = TRUE) # This will take forever
gstufis <- genetic_diversity(gstudred, stratum="pop", mode = "Fis", small.N = TRUE) # This will take forever
gstufpe <- genetic_diversity(gstudred, stratum="pop", mode = "Pe", small.N = TRUE) # This will take forever
gstuho[1:30,1:3]
identical(gstuhe[['Locus']],gstufis[['Locus']]) # Good

# Bind all results together
allstats <- cbind(gstua,gstuae$Ae,gstuhe$He,gstuho$Ho,gstufis$Fis,gstufpe$Pe)
allstats[1:10,]

# Change all infinite values to NA
any(is.infinite(allstats$`gstuae$Ae`)) # There are infinite values messing up the mean calcs
library(data.table)
for (j in 1:ncol(allstats)) set(allstats, which(is.infinite(allstats[[j]])), j, NA)

# Obtain means for each population
allstatspop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=mean, na.rm=TRUE, na.action=NULL)
colnames(allstatspop) <- c("Pop","A","Ae","He","Ho","Fis","Pe")
allstatspop[1:nrow(allstatspop),]

# Calculate number of SNPs used for deriving stats in each population
count_nas <- function(arg1) {
  return(sum(is.na(arg1)))
}
aggregate(allstats[,3:8],by=list(allstats$Stratum),FUN=count_nas) # There's big differences in # SNPs used for FIS calculations by population

# Obtain standard errors for each popn
library("plotrix")
allstatsSEpop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=std.error, na.rm=TRUE)
colnames(allstatsSEpop) <- c("Pop","ASE","AeSE","HeSE","HoSE","FisSE","PeSE")

# Cbind SEs to Means
allstatspop <- cbind(allstatspop,allstatsSEpop$ASE, allstatsSEpop$AeSE,allstatsSEpop$HeSE,allstatsSEpop$HoSE,allstatsSEpop$FisSE,allstatsSEpop$PeSE)
colnames(allstatspop) <- c("Pop","A","Ae","He","Ho","Fis","Pe","ASE","AeSE","HeSE","HoSE","FisSE","PeSE")

# Output to csv file
filename <- paste("output/popgenstats",currentDate,".csv",sep="")
write.csv(allstatspop,filename)


## Make boxplots of population genetic parameters
# Convert to ggplot format
library(reshape)
df <- melt(allstatspop, id.vars = c("Pop"))
df <- cbind(df[1:24,],df[25:48,])
df <- df[,c(1,2,3,6)]
colnames(df) <- c("Pop","Variable","Value","SE")
df <- df[!df$Variable=="A",] # Remove the parameters with big values to allow visual comparison of the more important parameters
df <- df[!df$Variable=="Ae",]

# Plot results
coni_params <- ggplot(data=df, aes(x=Variable, y=Value, fill=Pop, group=Pop)) +
  geom_col(position="dodge") +
  geom_errorbar(aes(ymin = Value-SE, ymax = Value+SE), width=0.2, size=0.2, position=position_dodge(0.9)) +
  # scale_fill_manual(values=alpha(c("#88CCEE", "#CC6677", "#DDCC77", "#117733"),0.9)) +
  scale_fill_manual(values=alpha(c("grey80", "grey60", "grey40", "grey20"),0.9)) +
  # geom_point(aes(fill=Variable), shape=21, position = position_jitterdodge(), size = 2) +
  theme_bw() +
  theme(panel.grid.major.y = element_blank()) +  
  theme(panel.grid.minor.y = element_blank()) +
  theme(panel.grid.major.x = element_blank()) +  
  theme(panel.grid.minor.x = element_blank()) +
  xlab("Parameter") + 
  ylab("Value") +
  theme(axis.title.y=element_text(size=13, margin=margin(0,16,0,0))) +
  theme(axis.title.x=element_text(size=13, margin=margin(16,0,0,0))) +
  theme(axis.text = element_text(size = 13)) +
  guides(fill=guide_legend(title="Population")) +
  theme(panel.border = element_blank()) +
  theme(axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5))

# Output
pdffilename <- paste("output/coni_pop_params_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", height=6, width=9)
coni_params
dev.off()






###########################
# Get pairwise FST matrix #
###########################

library(hierfstat)
pwPopFst <- genet.dist(genotypes, method="Nei87") # Pairwise Popn Nei Fst (corrects for sample sizes) - TAKES A FEW MINUTES
pwPopFst <- as.data.frame(as.matrix(pwPopFst))
pwPopFst[upper.tri(pwPopFst, diag=FALSE)] <- NA
# write.csv(pwPopFst,"output/population_fst_100820.csv",row.names=TRUE)
# write.csv(pwPopFst,"output/population_fst_NoRels_040221.csv",row.names=TRUE)

library(mmod) # Modern Methods of Differentiation
popgst <- pairwise_Gst_Hedrick(genotypes) # Calculates pairwise population values of G''st
# popgst <- Gst_Hedrick(genotypes) # Calculates per-locus and global Hedrick's G''st





############################################################
# Do spatial autocorrellation analysis using GenAlEx method #
############################################################

library(gstudio)

## Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))
geno.types <- congen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Remove island populations
genotypes <- genotypes[genotypes@pop!="Inglis Island",]
genotypes <- genotypes[genotypes@pop!="Melville Island",]
genotypes <- genotypes[genotypes@pop!="Bathurst Island",]

# Get genetic distance matrix
gendist <- dist(genotypes, method = "euclidean", diag = TRUE, upper = TRUE, p = 2)
gendist <- as.matrix(gendist)

# Get geographic distance matrix
sampinfo <- read.csv("output/samp_info_070820.csv")
sampinfo <- subset(sampinfo, sampinfo$sample %in% rownames(genotypes@tab))

# Check that they are in the same order as the rownames of congen
identical(sampinfo$sample,rownames(genotypes@tab))

library(fossil)
geogcoords <- sampinfo[c("long", "lat")]
geogdist <- earth.dist(geogcoords,dist=TRUE) # This is in km
geogdist <- as.matrix(geogdist)
colnames(geogdist) <- rownames(geogdist) <- rownames(genotypes@tab)

genauto <- genetic_autocorrelation(P=geogdist, G=gendist, bins=c(0,1,5,50,100,1000), perms=999, plot=TRUE)

library(ggplot2)

corrgram <- ggplot(data=genauto, aes(x=as.factor(To), y=R)) +
  geom_point(aes(), size=1.3) +
  geom_line(group=1, aes(), size=0.5) +
  geom_text_repel(aes(label=paste("P = ",round(P,digits=3),sep="")), force=1, point.padding=0.2, segment.color = "transparent") +
  geom_hline(yintercept = 0, size = 0.3, colour = "black", linetype = "dotted") +
  theme_bw() +
  theme(panel.grid.major.y = element_blank()) +  
  theme(panel.grid.minor.y = element_blank()) +
  theme(panel.grid.major.x = element_blank()) +  
  theme(panel.grid.minor.x = element_blank()) +
  xlab("End point of distance class (km)") + 
  ylab("Spatial autocorrelation") +
  theme(axis.title.y=element_text(margin=margin(0,10,0,0), colour = "black")) +
  theme(axis.title.x=element_text(margin=margin(10,0,0,0), colour = "black")) +
  theme(axis.text.x = element_text(colour="black")) +
  theme(axis.text.y = element_text(colour="black")) +
  theme(panel.border = element_blank()) +
  theme(legend.position = c(0.8, 0.8)) +
  theme(axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black",
                                                                                          size = 0.5))


# Output spatial autocorrelogram plot
pdffilename <- paste("output/coni_spatial_structure_mainland", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=7, height=4)
corrgram
dev.off()

















###########################################################
# Check population structure and map results using tess3r #
###########################################################

# See Caye et al. 2016 and 2018 for details of algorithms


## Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))

# tess3r package newer and better
#devtools::install_github("bcm-uga/TESS3_encho_sen")
library("tess3r")

## Prepare data
congen[1:10,1:10]
Y <- congen

## Create tess3r project

# Need lat and longs
allinfo <- read.csv(file="output/samp_info_070820.csv",header=TRUE,row.names=NULL) # 66 samples
allinfo <- subset(allinfo, allinfo$sample %in% rownames(congen))
identical(allinfo$sample,rownames(congen))
latlongs <- as.matrix(allinfo[,3:4])
latlongs <- latlongs[,c(2,1)]

# set up allinfo for later
yeahcats <- aggregate(allinfo[ ,3:4], list(allinfo$popn), mean)


# Runs (latent factors = ancestral populations??) K = 1:x with cross-entropy calcs, each done with y
# repetitions.
tess3.obj <- tess3(X = Y, coord = latlongs, K = 1:10, rep=10, # including rep just puts error bars onto scree plot
                   method = "projected.ls", ploidy = 2, openMP.core.num = 7)

# plot cross-entropy criterion of all runs of the project
plot(tess3.obj, pch = 19, col = "blue",
     xlab = "Number of ancestral populations",
     ylab = "Cross-validation score")


library(dplyr)
library(reshape)
library(ggplot2)

## Make admixture plot using K=2
# take the ancestry coefficients for the best run where K = 2
qmatrix2 <- qmatrix(tess3.obj, K = 2)
# format for ggplot
qmatrix2 <- as.data.frame(qmatrix2)
qmatrix2$popn <- popns
qmatrix2$sampname <- rownames(Y)
qmatrix2 <- qmatrix2[order(qmatrix2$popn),]
head(qmatrix2)
qmatrix2 <- melt(qmatrix2,id.vars=c("popn","sampname"))
qmatrix2$sampname <- as.factor(qmatrix2$sampname)
qmatrix2$sampname <- factor(qmatrix2$sampname, levels = qmatrix2$sampname[1:(0.5*nrow(qmatrix2))])


# Make plot for k2
q2 <- ggplot(qmatrix2, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#E69F00","#56B4E9","#009E73","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix2$popn)


## Make admixture plot using K=3
# get the cross-entropy of each run
qmatrix3 <- qmatrix(tess3.obj, K = 3)
# arrange for ggplot
qmatrix3 <- as.data.frame(qmatrix3)
qmatrix3$popn <- popns
qmatrix3$sampname <- rownames(Y)
qmatrix3 <- qmatrix3[order(qmatrix3$popn),]
head(qmatrix3)
qmatrix3 <- melt(qmatrix3,id.vars=c("popn","sampname"))
qmatrix3$sampname <- as.factor(qmatrix3$sampname)
qmatrix3$sampname <- factor(qmatrix3$sampname, levels = qmatrix3$sampname[1:(1/3*nrow(qmatrix3))])

# make plot for K4
q3 <- ggplot(qmatrix3, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#009E73","#E69F00","#56B4E9","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_text(angle = 90, size = 10)) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix3$popn)




### output plots K 2,3

# Organise layout of plots
lay <- rbind(c(1,1,1),
             c(1,1,1),
             c(1,1,1),
             c(1,1,1),
             c(1,1,1),
             c(2,2,2),
             c(2,2,2),
             c(2,2,2),
             c(2,2,2),
             c(2,2,2),
             c(2,2,2),
             c(2,2,2))

library(gridExtra)
pdffilename <- paste("output/tess3r_admix_plots_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", height=6, width=8)
grid.arrange(q2, q3, layout_matrix=lay)
dev.off()




### Map the results across the landscape

## K=2
# Use CreatePalette() to define color palettes.
my.colors <- c("#56B4E9","#009E73","#E69F00","blue3") #, "maroon1", "yellow", "tan1"
my.palette <- CreatePalette(my.colors, 7)

# Prepare q matrix for output
qmatrix2 <- qmatrix(tess3.obj, K = 2)
qmat <- as.data.frame(qmatrix2)
rownames(qmat) <- rownames(congen)
qmat$pops <- popns

# Output q matrix for k=x
csvfilename <- paste("qmatrix_k2_", currentDate,".csv",sep="")
write.csv(qmat, csvfilename)


#install.packages("rworldmap")
#install.packages("rworldxtra")
library(rworldmap)
library(rworldxtra)
library(ggplot2)
library(ggsn)

map.polygon <- getMap(resolution = "high")

admixturemap <- ggtess3Q(qmatrix2, latlongs, map.polygon=map.polygon, col.palette=my.palette) +
  geom_path(data = map.polygon, aes(x = long, y = lat, group = group), colour="grey40", size=0.4) +
  xlim(123, 137) + 
  ylim(-16,-10.9) + 
  coord_equal() + 
  geom_point(data = yeahcats, aes(x = long, y = lat), size = 0.6, colour="black") + 
  xlab("Longitude") +
  ylab("Latitude") + 
  theme_bw() +
  scale_alpha(guide = "none") +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black")) +
  scalebar(location="topleft",y.min=-16, y.max=-10.9, x.min=123, x.max=137, 
           dist=300, dist_unit="km", transform=TRUE, model='WGS84', st.dist=-0.05, st.size=3) +
  north(location = "topright", scale = 0.15, symbol = 1,
        y.min=-16, y.max=-10.9, x.min=123, x.max=137, anchor = NULL) +
  coord_quickmap()

admixturemap


## Output three admixture maps to pdfs
pdffilename <- paste("output/K2_admixture_map_tess3r", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=8, height=5)
admixturemap
dev.off()



## K=3
# Use CreatePalette() to define color palettes.
my.colors <- c("#56B4E9","#009E73","#E69F00","blue3") #, "maroon1", "yellow", "tan1"
my.palette <- CreatePalette(my.colors, 7)

# Prepare q matrix for output
qmatrix3 <- qmatrix(tess3.obj, K = 3)
qmat <- as.data.frame(qmatrix2)
rownames(qmat) <- rownames(congen)
qmat$pops <- popns

# Output q matrix for k=x
csvfilename <- paste("output/qmatrix_k3_", currentDate,".csv",sep="")
write.csv(qmat, csvfilename)


#install.packages("rworldmap")
#install.packages("rworldxtra")
library(rworldmap)
library(rworldxtra)
library(ggplot2)
library(ggsn)

map.polygon <- getMap(resolution = "high")

admixturemap <- ggtess3Q(qmatrix3, latlongs, map.polygon=map.polygon, col.palette=my.palette) +
  geom_path(data = map.polygon, aes(x = long, y = lat, group = group), colour="grey40", size=0.4) +
  xlim(123, 137) + 
  ylim(-16,-10.9) + 
  coord_equal() + 
  geom_point(data = yeahcats, aes(x = long, y = lat), size = 0.6, colour="black") + 
  xlab("Longitude") +
  ylab("Latitude") + 
  theme_bw() +
  scale_alpha(guide = "none") +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black")) +
  scalebar(location="topleft",y.min=-16, y.max=-11.5, x.min=123, x.max=137, 
           dist=300, dist_unit="km", transform=TRUE, model='WGS84', st.dist=-0.05, st.size=3) +
  north(location = "topright", scale = 0.15, symbol = 1,
        y.min=-16, y.max=-10.9, x.min=123, x.max=137, anchor = NULL) +
  coord_quickmap()

admixturemap


## Output three admixture maps to pdfs
pdffilename <- paste("output/K3_admixture_map_tess3r", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=8, height=5)
admixturemap
dev.off()











#########################################
# Output to GenAlEx for QDiver analysis #
#########################################

## Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))
geno.types <- congen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Remove the small (< 6 individuals) populations first
table(popns)
genotypes <- genotypes[genotypes@pop!="Cobourg West",]
genotypes <- genotypes[genotypes@pop!="Prince Regent",]
genotypes <- genotypes[genotypes@pop!="Inglis Island",]

# Impute/remove missingness
library(poppr)
genotypes <- missingno(genotypes, type = "loci", cutoff=0, quiet = FALSE, freq = FALSE)

# Convert to genalex format and output
# genind2genalex(genotypes,filename="output/coni_genos_genalex150221_missing.csv",overwrite=TRUE)
# genind2genalex(genotypes,filename="output/coni_genos_genalex210920.csv",overwrite=TRUE)
# genind2genalex(genotypes,filename="output/coni_genos_genalex061120.csv",overwrite=TRUE)









###########################################
## Calculate mean number of samples per SNP 

# Use genind made above
summary(genotypes@pop)

genos <- genotypes[genotypes@pop=="Cobourg East"]
genos <- genind2df(genos, pop = NULL, sep = "", usepop = FALSE, oneColPerAll = FALSE)
genos <- as.data.frame(t(genos))
genos$sampspersnp <- rowSums(is.na(genos))
genos$sampspersnp <- genos$sampspersnp/2
genos$sampspersnp <- 15-genos$sampspersnp
mean(genos$sampspersnp)

genos <- genotypes[genotypes@pop=="Melville Island"]
genos <- genind2df(genos, pop = NULL, sep = "", usepop = FALSE, oneColPerAll = FALSE)
genos <- as.data.frame(t(genos))
genos$sampspersnp <- rowSums(is.na(genos))
genos$sampspersnp <- genos$sampspersnp/2
genos$sampspersnp <- 11-genos$sampspersnp
mean(genos$sampspersnp)

genos <- genotypes[genotypes@pop=="Bathurst Island"]
genos <- genind2df(genos, pop = NULL, sep = "", usepop = FALSE, oneColPerAll = FALSE)
genos <- as.data.frame(t(genos))
genos$sampspersnp <- rowSums(is.na(genos))
genos$sampspersnp <- genos$sampspersnp/2
genos$sampspersnp <- 6-genos$sampspersnp
mean(genos$sampspersnp)


genos <- genotypes[genotypes@pop=="Mitchell Plateau"]
genos <- genind2df(genos, pop = NULL, sep = "", usepop = FALSE, oneColPerAll = FALSE)
genos <- as.data.frame(t(genos))
genos$sampspersnp <- rowSums(is.na(genos))
genos$sampspersnp <- genos$sampspersnp/2
genos$sampspersnp <- 10-genos$sampspersnp
mean(genos$sampspersnp)











#####################################################################
### Calculate niche hypervolumes and overlap for Conilurus lineages #
#####################################################################

# Clear environment and set up working dir and datestamp
rm(list = ls())
setwd("E:/conilurus_genetics")
currentDate <- Sys.Date()

# von Takach et al. 2020 found that Rainfall, Maxtemp, Mintemp were driving occurrence within the contemporary
# distribution, and that contemporary values of Elevation, Maxtemp, Mintemp, TWI, and TRI at occurrence points
# were significantly different to historical values. No impact of fire frequency per se on Conilurus observed.
# Radford et al. 2020 show that switch to early dry season burning positively impacts Conilurus.
# Work in the NT (Davies et al.) found similar with effects of fire.

library(raster)

# Read in raster covariate files
files <- list.files("output/rasters", pattern = "\\.tif$", full.names=TRUE)
files

# Stack covariates into a raster brick
predictors <- stack(files)
predictors

## Extract values from raster layers at coordinates of genotyped samples
library(adegenet)
# Get required info
load("output/coni_genind201020")
sampinfo <- read.csv("output/samp_info_070820.csv")
sampinfo <- subset(sampinfo, sampinfo$sample %in% rownames(coni_genind@tab))
genopoints <- sampinfo

# Convert to sp object
coordinates(genopoints) <- ~long + lat
crs(genopoints) <- "+proj=longlat +datum=WGS84"

# Extract values
genopoints <- extract(predictors, genopoints) # Extract niche values

# Combine dfs and make a column with the lineage name
sampinfo <- cbind(sampinfo, genopoints)
rownames(sampinfo) <- 1:nrow(sampinfo)
sampinfo$lineage <- "Mainland"
sampinfo$lineage[7:23] <- "Tiwis"
head(sampinfo)

# Scale and subset df into relevant lineages
sampinfo <- unique(sampinfo[, c("lat","long","annualrain","dem","maxtemp","mintemp","ruggedness","lineage")])
df <- sampinfo[,3:8]
df$annualrain <- as.numeric(scale(df$annualrain,center=TRUE,scale=TRUE))
df$dem <- as.numeric(scale(df$dem,center=TRUE,scale=TRUE))
df$maxtemp <- as.numeric(scale(df$maxtemp,center=TRUE,scale=TRUE))
df$mintemp <- as.numeric(scale(df$mintemp,center=TRUE,scale=TRUE))
df$ruggedness <- as.numeric(scale(df$ruggedness,center=TRUE,scale=TRUE))

mainland <- df[df$lineage=="Mainland" , ]
tiwis <- df[df$lineage=="Tiwis" , ]


## Get hypervolumes
library(hypervolume)

# Use SVM method because more comparable between subsets
nu <- 0.01
gamma <- 0.5

# mainland hypervolume
hv_ml <- hypervolume_svm(data = mainland[,1:5], name="mainland_svm", svm.nu = nu, svm.gamma = gamma, verbose=FALSE)
plot(hv_ml)

# Tiwi hypervolume
hv_ti <- hypervolume_svm(data = tiwis[,1:5], name="tiwis_svm", svm.nu = nu, svm.gamma = gamma, verbose=FALSE)
plot(hv_ti)

# Create hypervolume set
hset <- hypervolume_set(hv1=hv_ml, hv2=hv_ti, check.memory=FALSE)

# Check overlap between species
hypervolume_overlap_statistics(hset) # 0% overlap in niche

# Take a look at the volumes
volumes <- get_volume(hypervolume_join(hv_ml, hv_ti))
barplot(volumes)

write.csv(volumes,paste("output/svm_volumes_",currentDate,".pdf",sep=""),row.names=TRUE)

pdffilename <- paste("output/hypervolume_sizes_",currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=8, height=6)
barplot(volumes)
dev.off()



# Check pairwise plots between candidates and viable sites
pdffilename <- paste("output/SVM_hyp_",currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=9, height=8)
plot(hypervolume_join(hv_ml, hv_ti), contour.lwd=2, names=colnames(df)[1:4], cex.axis=1,
     colors = c("orange","blue"), show.random = FALSE, show.centroid = TRUE, show.legend = TRUE, point.dark.factor=0.1, cex.data=0.75)
dev.off()
# Holes in boundaries are likely due to low number of datapoints, I think where the hypervolume boundary is uncertain


formatC(volumes, format = "e", digits = 2)
paste(mean(sampinfo$annualrain[sampinfo$lineage=="Mainland"]),"(+-",sd(sampinfo$annualrain[sampinfo$lineage=="Mainland"]),")", sep="")
paste(mean(sampinfo$annualrain[sampinfo$lineage=="Tiwis"]),"(+-",sd(sampinfo$annualrain[sampinfo$lineage=="Tiwis"]),")", sep="")






























#####################################################################################################


#####################################################################################################


############################################################


## Old code






########################################
# Check Structure-type plots using LEA #
########################################

## Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))
rowSums(is.na(congen))


# Change name to avoid filtering if you mess up somewhere
Y <- congen

# BiocManager::install("LEA")
library(LEA)

# Format for LEA package. Use cross-entropy K value
Y[1:10,1:10] # Need to deal with missing data. Change to 9 (could probably impute somehow too)
Y[is.na(Y)] <- 9
Y[1:10,1:10]

Y <- as.data.frame(t(Y)) # Make sure it's one row for each SNP, as geno should be!!


# Create stupid geno file (can be used for imputation)
#write.geno(Y, "genotypes.geno") # If doing this method, Y needs to be SNPs in columns
write.table(Y, "output/genotypes140920.geno", row.names=FALSE, col.names=FALSE, sep="")

# Runs (latent factors = ancestral populations) K = 1:x with cross-entropy calcs, each done with y repetitions.
project1 = NULL
project1 = snmf("output/genotypes140920.geno", K=1:10, entropy = TRUE, repetitions = 5, CPU=1,
                project = "new", ploidy=2) # Takes a few mins

project1 = snmf("output/genotypes140920.geno", K=4, entropy = TRUE, repetitions = 3, CPU=1,
                project = "new", ploidy=2) # Takes a few mins

# Load project
project2 = load.snmfProject("output/genotypes140920.snmfProject")

# plot cross-entropy criterion of all runs of the project
plot(project2, lwd = 5, col = "red", pch=1)



library(dplyr)
library(reshape)
library(ggplot2)

## Make admixture plot using K=2
# get the cross-entropy of each run
ce = cross.entropy(project2, K = 2)
# select the run with the lowest cross-entropy
best = which.min(ce)
# take the ancestry coefficients for the best run where K = 2
qmatrix2 = Q(project2, K = 2, run = best)
# format for ggplot
qmatrix2 <- as.data.frame(qmatrix2)
qmatrix2$popn <- popns
qmatrix2$sampname <- colnames(Y)
qmatrix2 <- qmatrix2[order(qmatrix2$popn),]
head(qmatrix2)
qmatrix2 <- melt(qmatrix2,id.vars=c("popn","sampname"))
qmatrix2$sampname <- as.factor(qmatrix2$sampname)
qmatrix2$sampname <- factor(qmatrix2$sampname, levels = qmatrix2$sampname[1:51])


# Make plot for k2
q2 <- ggplot(qmatrix2, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#E69F00","#56B4E9","#009E73","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix2$popn)


## Make admixture plot using K=3
# get the cross-entropy of each run
ce = cross.entropy(project2, K = 3)
# select the run with the lowest cross-entropy
best = which.min(ce)
# take the ancestry coefficients for the best run where K = 3
qmatrix3 = Q(project2, K = 3, run = best)
# arrange for ggplot
qmatrix3 <- as.data.frame(qmatrix3)
qmatrix3$popn <- popns
qmatrix3$sampname <- colnames(Y)
qmatrix3 <- qmatrix3[order(qmatrix3$popn),]
head(qmatrix3)
qmatrix3 <- melt(qmatrix3,id.vars=c("popn","sampname"))
qmatrix3$sampname <- as.factor(qmatrix3$sampname)
qmatrix3$sampname <- factor(qmatrix3$sampname, levels = qmatrix3$sampname[1:51])

# make plot for K3
q3 <- ggplot(qmatrix3, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#009E73","#E69F00","#56B4E9","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix3$popn)


## Make admixture plot using K=4
# get the cross-entropy of each run
ce = cross.entropy(project2, K = 4)
# select the run with the lowest cross-entropy
best = which.min(ce)
# take the ancestry coefficients for the best run where K = 3
qmatrix4 = Q(project2, K = 4, run = best)
# arrange for ggplot
qmatrix4 <- as.data.frame(qmatrix4)
qmatrix4$popn <- popns
qmatrix4$sampname <- colnames(Y)
qmatrix4 <- qmatrix4[order(qmatrix4$popn),]
head(qmatrix4)
qmatrix4 <- melt(qmatrix4,id.vars=c("popn","sampname"))
qmatrix4$sampname <- as.factor(qmatrix4$sampname)
qmatrix4$sampname <- factor(qmatrix4$sampname, levels = qmatrix4$sampname[1:51])


# make plot for K4
q4 <- ggplot(qmatrix4, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#009E73","#E69F00","#56B4E9","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_text(angle = 90, size = 10)) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix4$sampname)


### output plots K 2,3, and 4

# Organise layout of plots
lay <- rbind(c(1,1,1),
             c(1,1,1),
             c(2,2,2),
             c(2,2,2),
             c(3,3,3),
             c(3,3,3),
             c(3,3,3))

library(gridExtra)
pdffilename <- paste("output/admixture_plots_280820", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", height=8, width=8)
grid.arrange(q2, q3, q4, layout_matrix=lay)
dev.off()










#############################################
# Check Structure-type plots using NGSadmix #
#############################################

## Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))
geno.types <- congen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Sort out admixture files from NGSadmix output
k2 <- read.table("output/ngsadmix_out_k2.qopt")
k3 <- read.table("output/ngsadmix_out_k3.qopt")
k4 <- read.table("output/ngsadmix_out_k4.qopt")
samps <- read.csv(file="output/samp_info_070820.csv",header=TRUE,row.names=NULL) # 66 samples
k2$sampname <- samps$sample
k3$sampname <- samps$sample
k4$sampname <- samps$sample
k2 <- subset(k2, k2$sampname %in% rownames(genotypes@tab))
k3 <- subset(k3, k3$sampname %in% rownames(genotypes@tab))
k4 <- subset(k4, k4$sampname %in% rownames(genotypes@tab))
identical(k2$sampname, rownames(genotypes@tab))
k2$popn <- genotypes@pop
k3$popn <- genotypes@pop
k4$popn <- genotypes@pop
qmatrix2 <- k2
qmatrix3 <- k3
qmatrix4 <- k4

# Change to ggplot format and order correctly
qmatrix2 <- qmatrix2[order(qmatrix2$popn),]
head(qmatrix2)
qmatrix2 <- melt(qmatrix2,id.vars=c("popn","sampname"))
qmatrix2$sampname <- as.factor(qmatrix2$sampname)
qmatrix2$sampname <- factor(qmatrix2$sampname, levels = qmatrix2$sampname[1:51])

qmatrix3 <- qmatrix3[order(qmatrix3$popn),]
head(qmatrix3)
qmatrix3 <- melt(qmatrix3,id.vars=c("popn","sampname"))
qmatrix3$sampname <- as.factor(qmatrix3$sampname)
qmatrix3$sampname <- factor(qmatrix3$sampname, levels = qmatrix3$sampname[1:51])

qmatrix4 <- qmatrix4[order(qmatrix4$popn),]
head(qmatrix4)
qmatrix4 <- melt(qmatrix4,id.vars=c("popn","sampname"))
qmatrix4$sampname <- as.factor(qmatrix4$sampname)
qmatrix4$sampname <- factor(qmatrix4$sampname, levels = qmatrix4$sampname[1:51])

# Make plot for k2
q2 <- ggplot(qmatrix2, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#009E73","#56B4E9","#E69F00","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix2$popn)

# make plot for K3
q3 <- ggplot(qmatrix3, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#009E73","#E69F00","#56B4E9","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix3$popn)

# make plot for K4
q4 <- ggplot(qmatrix4, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#56B4E9","#F0E442","#E69F00","#009E73")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_text(angle = 90, size = 10)) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix4$popn)


### output plots K 2,3, and 4

# Organise layout of plots
lay <- rbind(c(1,1,1),
             c(1,1,1),
             c(2,2,2),
             c(2,2,2),
             c(3,3,3),
             c(3,3,3),
             c(3,3,3))

library(gridExtra)
pdffilename <- paste("output/NGSadmix_plots_140920", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", height=8, width=8)
grid.arrange(q2, q3, q4, layout_matrix=lay)
dev.off()









### Dendrogram using SNPRelate
## Need to load the gds object first (not shown)

# Individual tree
diss <- snpgdsDiss(gds)
hc <- snpgdsHCluster(diss)
# split
set.seed(100)
rv <- snpgdsCutTree(hc, label.H=TRUE, label.Z=TRUE)
# draw dendrogram
snpgdsDrawTree(rv, main="Conilurus",
               edgePar=list(col=rgb(0.5,0.5,0.5, 0.75), t.col="black"))





##########################
# Output to Ne Estimator #
##########################

## Read in congen dataset
congen <- read.csv("output/congen140920.csv", header=TRUE, row.names=1)
popns <- read.csv("output/congen_popns_140920.csv")
popns <- popns$x
congen <- as.data.frame(t(congen))
geno.types <- congen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Subset to popn of interest
genotypesInglis <- genotypes[genotypes@pop=="Inglis Island",]
genotypesMPOld <- genotypes[c("CPE0056","CPE0057","CPE0058","CPE0059","CPE0060","CPE0061"),]
genotypesMPNew <- genotypes[c("CPE0051","CPE0052","CPE0054"),]
genotypesBath <- genotypes[genotypes@pop=="Bathurst Island",]
genotypesMelville <- genotypes[genotypes@pop=="Melville Island",]
genotypes <- genotypes[genotypes@pop=="Cobourg East",]

# remotes::install_github("romunov/zvau")
library(remotes)
writeGenPop(genotypes, "output/genepopnest","bvt") # Cobourg East
writeGenPop(genotypesInglis, "output/genepopnestInglis","bvt_Inglis") # Inglis Island
writeGenPop(genotypesMPOld, "output/genepopnestMPOld","bvt_MPOld") # Mitchell Plateau Old
writeGenPop(genotypesMPNew, "output/genepopnestMPNew","bvt_MPNew") # Mitchell Plateau New
writeGenPop(genotypesMelville, "output/genepopnestMelville","bvt_Melville") # Melville Island


