#!/usr/bin/env ruby
require "fileutils"

# change dir to home folder: https://ruby-doc.org/core-2.7.1/Dir.html
# to find the path to your folder can type: pwd
Dir.chdir("/data")

# Make sure the output folders exist
FileUtils.mkdir_p "./coni_bam_filt/"
FileUtils.mkdir_p "./coni_bam_sort/"

# get an array of files to use
bamfiles = Dir.glob(File.join("**", "coni_bams", "*.bam"))

# run through the files filtering out unmapped reads
bamfiles.each do |bam_file_path|
  # gsub is a find and replace
  output_filename = bam_file_path.gsub("coni_bams", "coni_bam_filt").gsub(".bam", ".bam")
  %x[samtools view -h -F 4 -b #{bam_file_path} > #{output_filename}]
end

# get a new array of files to use
bamfiles = Dir.glob(File.join("**", "coni_bam_filt", "*.bam"))

# run through the files sorting the reads
bamfiles.each do |bam_file_path|
  # gsub is a find and replace
  output_filename = bam_file_path.gsub("coni_bam_filt", "coni_bam_sort").gsub(".bam", ".bam")
  %x[samtools sort -o #{output_filename} #{bam_file_path}]
end

# get a new array of files to use
bamfiles = Dir.glob(File.join("**", "coni_bam_sort", "*.bam"))

# run through the files and index each file
bamfiles.each do |bam_file_path|
  # gsub is a find and replace
  output_filename = bam_file_path.gsub("coni_bam_sort", "coni_bam_sort").gsub(".bam", ".bai")
  %x[samtools index #{bam_file_path} #{output_filename}]
end